import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

/**
 * Write a description of class Canvas here.
 * 
 * @author Taylor Born
 * @version January 2011
 */
public class Canvas  extends World
{
    private static GreenfootSound backSound = new GreenfootSound("Breaks.mp3");
    private boolean playingSound = true;
    private int setting = 0;
    private String[] messages = { "Life's Ripple",
                                  "Turbine",
                                  "Time and Space",
                                  "Broken Shell",
                                  "Pin Toy",
                                  "Helix",
                                  "Black Hole",
                                  "Jelly Rings",
                                  "Blazing Sun",
                                  "Imagination Ladder"
                                };
    
    private String message = messages[0];
    private int fadeMessage = 210;
    private String keysPressed = "";
    
    private static final int CODE_ID = 44;
    private boolean[] visited = new boolean[messages.length];
    private String code;
    
    //byte[] waveBytes;

    //boolean played = false;
    
    //long previous;
    //long current;
    //long difference;
    //long start;
    
    public Canvas()
    {
        super(600, 400, 1);
        CodeKeyUser.enter();
        code = CodeGenerator.getCode(CODE_ID); // needs to be after the above statement
        visited[0] = true;
//         try
//         {
//             FileInputStream waveFileStream = new FileInputStream("sound.wav");
//             waveBytes = new byte[waveFileStream.available()];
//             waveFileStream.read(waveBytes);
//         }
//         catch (Exception e)
//         {
//             System.err.println(e);
//         }
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        setBackground(pic);
    }
    public void draw()
    {
        switch (setting)
        {
            case 0: drawRipple(); break;
            case 1: drawWater(); break;
            case 2: drawTime(); break;
            case 3: drawShell(); break;
            case 4: drawPin(); break;
            case 5: drawHelix(); break;
            case 6: drawHole(); break;
            case 7: drawJelly(); break;
            case 8: drawSun(); break;
            case 9: drawNew(); break;
        }
    }
    public void stopped()
    {
        backSound.pause();
    }
    public void started()
    {
        if (playingSound)
            backSound.playLoop();
    }
    public void act()
    {
        if (Greenfoot.isKeyDown("left"))
        {
            if (!keysPressed.contains(" left "))
            {
                if (setting > 0)
                {
                    message = messages[--setting];
                    fadeMessage = 300;
                }
                keysPressed += " left ";
            }
        }
        else
            keysPressed = keysPressed.replace(" left ", "");
        
        if (Greenfoot.isKeyDown("right"))
        {
            if (!keysPressed.contains(" right "))
            {
                if (setting < messages.length - 1)
                {
                    message = messages[++setting];
                    fadeMessage = 300;
                }
                keysPressed += " right ";
            }
        }
        else
            keysPressed = keysPressed.replace(" right ", "");
        
        visited[setting] = true;
        
        draw();
        if (Greenfoot.isKeyDown("s"))
        {
            if (!keysPressed.contains(" s "))
            {
                keysPressed += " s ";
                playingSound = !playingSound;
                if (playingSound)
                {
                    backSound.playLoop();
                    message = "Sound On";
                }
                else
                {
                    backSound.pause();
                    message = "Sound Off";
                }
                fadeMessage = 300;
            }
        }
        else
            keysPressed = keysPressed.replace(" s ", "");
        if (fadeMessage > 0)
            fadeMessage -= 2;
        GreenfootImage text = new GreenfootImage(message, 16, Color.WHITE, Color.BLACK);
        GreenfootImage pic = new GreenfootImage(text.getWidth() + 24, text.getHeight() + 12);
        Graphics g = pic.getAwtImage().getGraphics();
        g.setColor(Color.BLACK);
        g.fillRoundRect(0, 0, pic.getWidth() - 1, pic.getHeight() - 1, 15, 15);
        pic.drawImage(text, 12, 6);
        pic.setTransparency(fadeMessage > 180 ? 180 : fadeMessage);
        getBackground().drawImage(pic, 10, 370);
        if (CodeGenerator.haveUser())
        {
            boolean flag = false;
            for (int i = 0; i < visited.length; i++)
                if (!visited[i])
                {
                    flag = true;
                    break;
                }
            if (!flag)
            {
                getBackground().setColor(new Color(255, 255, 255, fadeMessage > 180 ? 180 : fadeMessage));
                getBackground().drawString("Enter this code into bourne's Residents scenario: " + code, 5, 13);
            }
        }
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      LIFE'S RIPPLE
     */
    private int degreeR;
    
    public void drawRipple()
    {
        degreeR += 5;
        if (degreeR == 360)
            degreeR = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        pic.setColor(Color.RED);
        for (int i = 0; i < getWidth(); i += 5)
        {
            double rad = (degreeR + i) * Math.PI / 180d;
            int y = 200 + (int)(Math.sin(rad) * 30d);
            if (i % 15 == 0)
                pic.drawLine(i, y, i + (int)(Math.cos((degreeR + i) % 360 * Math.PI / 180d) * 25), y + (int)(Math.sin((degreeR + i) % 360 * Math.PI / 180d) * 25));
            else
                pic.setColorAt(i, y, Color.BLUE);
        }
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      TURBINE
     */
    private int degreeW;
    
    public void drawWater()
    {
        degreeW += 10;
        if (degreeW == 360)
            degreeW = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        int m = degreeW + 360 * 8;
        for (int i = m; i >= 0; i -= 30)
        {
            double rad = (i - degreeW) * Math.PI / 180d;
            GreenfootImage p = new GreenfootImage(1 + i / 20, 1 + i / 20);
            p.setColor(new Color(0, 0, (int)(255d * (double)(m - i) / m)));
            p.fillOval(0, 0, p.getWidth() - 1, p.getHeight() - 1);
            p.setTransparency((int)(255d * (double)(m - i) / m));
            int x = 300 + (int)(Math.cos(rad) * i / 10d);
            int y = 200 + (int)(Math.sin(rad) * i / 10d);
            pic.drawImage(p, x - p.getWidth() / 2 + (int)(Math.cos(rad - Math.PI / 180) * (-i % 180)), y - p.getHeight() / 2 + (int)(Math.sin(rad - Math.PI / 180) * (-i % 180)));
        }
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      TIME AND SPACE
     */
    private int degreeT;
    
    public void drawTime()
    {
        if (++degreeT == 360)
            degreeT = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        for (int i = 1 + degreeT; i < degreeT + 3600; i++)
        {
            pic.setColor(new Color(0, (int)((i % 50d) / 80d * 255d), 0));
            int x = 300 - i % 100 + (int)(Math.cos((i - degreeT) * Math.PI / 180) * i * .1) + i / 40;
            int y = 200 - i % 100 + (int)(Math.sin((i - degreeT) * Math.PI / 180) * i * .1);
            pic.drawRect(x, y, (int)(15 * i % 15), (int)(i % 30 * Math.sin(degreeT * Math.PI / 180)));
        }
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      BROKEN SHELL
     */
    private int degreeS;
    
    public void drawShell()
    {
        if (++degreeS == 360)
            degreeS = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        Graphics g = pic.getAwtImage().getGraphics();
        for (int i = 0; i < 220; i += 10)
        {
            int a = (int)(i / 220d * 720d);
            int c = (int)((i / 220d) * 255);
            int x = 300 - i / 2 + (int)(Math.cos((degreeS + a) * Math.PI / 180d) * i % 220);
            int y = 200 - i / 2 + (int)(Math.sin((degreeS + a) * Math.PI / 180d) * i % 220);
            int size = i * 3 / 2;
            if (i > 0)
            {
                g.setColor(new Color(0, 0, c));
                g.drawLine(x + size / 2, y + size / 2, 300 - (i - 10) / 2 + (int)(Math.cos((degreeS + (int)((i - 10) / 220d * 720d)) * Math.PI / 180d) * (i - 10) % 220) + (i - 10) * 3 / 2 / 2, 200 - (i - 10) / 2 + (int)(Math.sin((degreeS + (int)((i - 10) / 220d * 720d)) * Math.PI / 180d) * (i - 10) % 220) + (i - 10) * 3 / 2 / 2);
            }
            g.setColor(new Color(c, c, 0));
            g.fillArc(x, y, size, size, degreeS + a, 10 + degreeS / 180 + a / 12 - (int)((220 - i) / 220d * 90d));
        }
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      Pin Toy
     */
    private double numSquare;
    
    public void drawPin()
    {
        numSquare += .1d;
        if (numSquare > 360)
            numSquare = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        for (int i = 200; i < 400; i++)
            for (int k = 100; k < 300; k++)
                if ((int)((k + i) * Math.sin((numSquare * (i - 200) / 200d * (k - 100) / 200d) * Math.PI / 180d)) % 5 == (int)(Math.sqrt(Math.pow(i % 3 - k % 3, 2) + Math.pow(k % 3 - i % 3, 2))) % 5)
                    pic.setColorAt(i, k, Color.RED);
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      Helix
     */
    private double numHelix;
    
    public void drawHelix()
    {
        numHelix += 3d;
        if (numHelix > 720)
            numHelix = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        pic.setColor(Color.BLUE);
        for (int i = 0; i < 41; i++)
        {
            for (int k = 75; k >= -75; k -= 5)
            {
                int d = (int)(Math.sin(((numHelix / 2 + i * 18d) * Math.PI / 180d)) * k) + 75;
                int x = 300 + (int)(Math.cos(((numHelix / 2 + i * 18d) * Math.PI / 180d)) * k) - 2;
                pic.setColor(new Color(0, 0, (int)((150 - d) / 150d * 255d))); //d > 75 ? 0 : (int)((d / 75d) * 255d)
                pic.fillOval(x, i * 10 - 2, 4, 4);
            }
        }
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      Black Hole
     */
    private double numHole;
    
    public void drawHole()
    {
        numHole += 10;
        if (numHole > 1080)
            numHole = 0;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        
        pic.setColor(Color.YELLOW);
        int x = 300 + (int)(Math.cos(numHole / 3 * Math.PI / 180) * 100);
        int y = 200 + (int)(Math.sin(numHole / 3 * Math.PI / 180) * 100);
        
        int x2 = x + (int)(Math.cos((-numHole + 80) * Math.PI / 180) * 80);
        int y2 = y + (int)(Math.sin((-numHole + 80) * Math.PI / 180) * 80);
        
        for (int i = 0; i < 360; i += 6)
        {
            Point p1 = new Point(x + (int)(Math.cos(i * Math.PI / 180) * 100) + (int)(Math.cos((-numHole + (i % 30) * 12) * Math.PI / 180) * 40), y + (int)(Math.sin(i * Math.PI / 180) * 100) + (int)(Math.sin((-numHole + (i % 30) * 12) * Math.PI / 180) * 40));
            Point p2 = new Point(x2 + (int)(Math.cos((i + 50) * Math.PI / 180) * 60), y2 + (int)(Math.sin((i + 50) * Math.PI / 180) * 60));
            int n = (int)(Math.sqrt(Math.pow(p1.getX() - p2.getX(), 2) + Math.pow(p1.getY() - p2.getY(), 2)) / 300d * 255);
            pic.setColor(new Color(n, 0, 0, 100));
            pic.drawLine((int)p1.getX(), (int)p1.getY(), (int)p2.getX(), (int)p2.getY());
        }
        
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      Jelly Rings
     */
    private double numRing;
    
    public void drawJelly()
    {
        numRing += 6;
        if (numRing > 720)
            numRing -= 720;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        pic.setColor(Color.GREEN);
        for (int i = 0; i < 720; i += 10)
        {
            int x = 300 + (int)(Math.cos(i * Math.PI / 180) * (i + Math.sin((-numRing + i) * Math.PI / 180) * 150) / 2);
            int y = 200 + (int)(Math.sin(i * Math.PI / 180) * (i + Math.sin((-numRing + i) * Math.PI / 180) * 150) / 2);
            int w = (int)(Math.sin((numRing + i) * Math.PI / 180) * 20);
            pic.drawOval(x - w / 2, y - w / 2, w, w);
        }
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      Blazing Sun
     */
    private double numSun;
    
    public void drawSun()
    {
        numSun += 3;
        if (numSun > 720)
            numSun -= 720;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        
        pic.setColor(Color.ORANGE);
        for (int i = 0; i < 360; i++)
        {
            int max = (int)Math.abs(numSun - 360);
            
            double d = (numSun % (i % 5)) / (i + 1) + 1;
            Point p = new Point(300 + (int)(Math.cos(i * Math.PI / 180) * d), 200 + (int)(Math.sin(i * Math.PI / 180) * d));
            
            boolean dr = true;
            
            max -= d;
            while (max > 0)
            {
                d = Math.abs(d % (numSun + i + 1) / (1 + Math.cos(numSun * Math.PI / 180) * 5) + 10);
                if (d > max)
                    d = max;
                Point pnt = new Point((int)(p.getX() + Math.cos(i * Math.PI / 180) * d), (int)(p.getY() + Math.sin(i * Math.PI / 180) * d));
                pnt.translate((int)(Math.cos(((numSun + i) % 360) * Math.PI / 180) * 5), (int)(Math.sin(((numSun + i) % 360) * Math.PI / 180) * 5));
                if (dr)
                    pic.drawLine((int)p.getX(), (int)p.getY(), (int)pnt.getX(), (int)pnt.getY());
                dr = !dr;
                max -= d;
                p = pnt;
            }
        }
        
        setBackground(pic);
    }
    /* ################### # # # # # # # ###################
     * ######################## # # ########################
     * ################### # # # # # # # ###################
     * 
     *      Imagination Ladder
     */
    private double numX;
    
    public void drawNew()
    {
        numX += 1;
        if (numX > 720)
            numX -= 720;
        GreenfootImage pic = new GreenfootImage(600, 400);
        pic.fill();
        
        for (int i = 0; i < 360; i++)
        {
            pic.setColor(new Color((int)(Math.sin(Math.toRadians(numX % 60 + i % 30)) * 255), (int)(Math.cos(Math.toRadians(numX % 30 + i % 60)) * 255), (int)(Math.sin(Math.toRadians(numX % 90 + i % 15)) * 255), 150));
            int w = (int)(Math.cos(Math.toRadians(i * numX)) * 45);
            pic.fillOval((int)(Math.cos(Math.toRadians(numX + i)) * 50 + Math.abs(numX + i - 540) / 540d * 600 + Math.cos(Math.toRadians((numX * i) % 360)) * w) - w / 2, (int)(Math.sin(Math.toRadians(numX + i)) * 50 + Math.abs(numX + i - 540) / 540d * 400 + Math.sin(Math.toRadians((numX * i) % 360)) * w) - w / 2, w, w);
        }
        
        setBackground(pic);
    }
}